@@ -3,6 +3,7 @@ import sys, os
 from pathlib import Path
 from collections import defaultdict, OrderedDict
 from c3linearize import linearize
+import importlib.util
 
 from omnibelt import get_printer, load_yaml
 
@@ -13,6 +14,7 @@ from .errors import UnknownArtifactError, artifact_errors, MissingModifierError,
 from .registry import Script_Registry, Component_Registry, Modifier_Registry, Config_Registry
 from .external import include_files, include_package, register_project_type
 from .util import global_settings, configurize, parse_arg
+from . import projects as projects_module
 
 prt = get_printer(__name__)
 
@@ -76,6 +78,7 @@ class Workspace(Container):
 	# region Registration
 	
 	def reset_registries(self):
+		'''Clears all registries'''
 		if not self.silent:
 			prt.debug(f'Resetting registries of {self}')
 		self.scripts = Script_Registry()
@@ -87,6 +90,7 @@ class Workspace(Container):
 		                        modifier=self.modifiers, config=self.configs)
 	
 	def register_artifact(self, atype, name, info):
+		'''General function to register an artifact of type `atype` with name `name`'''
 		
 		registry = self._registries.get(atype, None)
 		if registry is None:
@@ -188,13 +192,21 @@ class Workspace(Container):
 	# region Artifacts
 	
 	def has_artifact(self, atype, name):
+		'''Check if this workspace has an artifact of type `atype` registered with name `name`'''
 		registry = self._registries.get(atype, None)
 		if registry is None:
 			raise UnknownArtifactError(atype)
 		return name in registry
 	
 	def find_artifact(self, atype, name):
+		'''
+		Find a registered artifact from the type and name
 		
+		:param atype: component, modifier, script, or config
+		:param name: registered name
+		:return: artifact entry, throws UnknownArtifactError if `atype` is not recognized,
+		and the corresponding artifact missing error if the name cannot be found
+		'''
 		registry = self._registries.get(atype, None)
 		if registry is None:
 			raise UnknownArtifactError(atype)
@@ -207,6 +219,7 @@ class Workspace(Container):
 		return artifact
 
 	def view_artifacts(self, atype):
+		'''Return a shallow copy of the full registry of type `atype`'''
 
 		registry = self._registries.get(atype, None)
 		if registry is None:
@@ -246,6 +259,7 @@ class Workspace(Container):
 		except MissingConfigError:
 			path = Path(name)
 			if path.suffix in {'.yaml', '.yml'} and path.is_file():
+				return Config_Registry.default(str(path))
 				return str(path)
 			
 			if not path.is_dir():
@@ -258,13 +272,15 @@ class Workspace(Container):
 			if path.is_dir():
 				cpath = path / 'config.yaml'
 				if cpath.is_file():
+					return Config_Registry.default(str(cpath))
 					return str(cpath)
 				cpath = path / 'config.yml'
 				if cpath.is_file():
+					return Config_Registry.default(str(cpath))
 					return str(cpath)
 			
 			raise
-		return entry.path
+		return entry
 	def view_configs(self):
 		return self.view_artifacts('config')
 	
@@ -465,7 +481,7 @@ class Workspace(Container):
 		
 		return merged
 	
-	def _process_single_config(self, data, process=True, parents=None):
+	def _process_single_config(self, data, process=True, parents=None, tree=None, me=''):
 		'''
 		This loads the data (if a path or name is provided) and then checks for parents and loads those as well
 
@@ -479,18 +495,28 @@ class Workspace(Container):
 		
 		if isinstance(data, str):
 			data = self.find_config(data)
-			data = self._load_config_from_path(data, process=process)
+		if isinstance(data, Config_Registry.entry_cls):
+			me = data
+			data = self._load_config_from_path(data.path, process=process)
 		
 		if parents is not None and 'parents' in data:
 			todo = []
+			if tree is not None:# and me not in edges:
+				tree[me] = []
 			for parent in data['parents']:  # prep new parents
 				# ppath = _config_registry[parent] if parent in _config_registry else parent
-				ppath = self.find_config(parent)
-				if ppath not in parents:
-					todo.append(ppath)
-					parents[ppath] = None
-			for ppath in todo:  # load parents
-				parents[ppath] = self._process_single_config(ppath, parents=parents)
+				entry = self.find_config(parent)
+				# ppath = entry.path
+				if entry.path not in parents:
+					todo.append(entry)
+					parents[entry] = None
+				if tree is not None:
+					tree[me].append(entry)
+			for entry in todo:  # load parents
+				proj = self if entry.project is None else entry.project
+				parents[entry] = proj._process_single_config(entry, parents=parents, tree=tree)
+		elif tree is not None:
+			tree[me] = []
 		
 		return data
 	
@@ -549,40 +575,30 @@ class Workspace(Container):
 		root['parents'] = configurize(data=reg + (list(root['parents']) if 'parents' in root else []))
 		
 		parents = {}
+		tree = {}
 		
-		root = self._process_single_config(root, parents=parents)
+		root_id = ''
+		root = self._process_single_config(root, parents=parents, tree=tree, me=root_id)
 		
 		pnames = []
 		if len(parents):  # topo sort parents
 			
-			# TODO: maybe clean up?
+			order = linearize(tree, heads=[root_id], order=True)[root_id]
 			
-			root_id = ' root'
-			src = defaultdict(list)
-			
-			names = {self.find_config(p): p for p in root['parents']} if 'parents' in root else {}
-			src[root_id] = list(names.keys())
-			
-			for n, data in parents.items():
-				connections = {self.find_config(p): p for p in data['parents']} if 'parents' in data else {}
-				names.update(connections)
-				src[n] = list(connections.keys())
-			
-			order = linearize(src, heads=[root_id], order=True)[root_id]
-			
-			pnames = [names[p] for p in order[1:]]
+			pnames = [(node.name if node.project is None or not isinstance(node.project, Project)
+			           else f'{node.project}:{node.name}') for node in order[1:]]
 			order = [root] + [parents[p] for p in order[1:]]
 			
 			# for analysis, record the history of all loaded parents
 			order = list(reversed(order))
 		
-		else:  # TODO: clean up
+		else:
 			order = [root]
 		
 		root = self._merge_configs(order, )
 		root.set_project(self)
 		
-		include_history = root.pull('_history_key', None, silent=True)
+		include_history = root.pull('_ancestry_key', 'ancestors', silent=True)
 		if include_history is not None:
 			root.push(include_history, pnames, silent=True)
 		
@@ -663,25 +679,30 @@ class Project(Workspace):
 		# region path
 		
 		self.info_path = raw.get('info_path', None)
-		self.root = os.path.dirname(self.info_path)
+		self.root = Path(self.info_path).parents[0]
 		
 		# endregion
 		
 		if 'py_info' in raw:
-			info = {'__file__': os.path.join(self.root, raw['py_info'])}
+			info = {'__file__': str(self.root / raw['py_info'])}
 			with open(info['__file__'], 'r') as f:
 				exec(f.read(), info)
 			del info['__file__']
 			raw.update(info)
+			info.update(raw)
+			raw = info
 		
 		# region info
 		
 		self.name = raw.get('name', None)
 		
-		self.author = raw.get('author', None)
+		author = raw.get('author', None)
+		self.primary_author = raw.get('primary_author', author)
 		self.authors = raw.get('authors', None)
-		if self.authors is None and self.author is not None:
-			self.authors = [self.author]
+		if self.authors is None and self.primary_author is not None:
+			self.authors = [self.primary_author]
+		if self.primary_author is None and self.authors is not None and len(self.authors):
+			self.primary_author = self.authors[0]
 		
 		self.github = raw.get('github', None)
 		self.url = raw.get('url', None)
@@ -690,19 +711,19 @@ class Project(Workspace):
 		
 		self.version = raw.get('version', None)
 		self.license = raw.get('license', None)
-		self.use = raw.get('use', 'leaf')  # {leaf, package}
+		# self.use = raw.get('use', 'leaf')  # {leaf, package}
 		
 		self.add_to_path = raw.get('add_to_path', True)
 		
 		self.description = raw.get('description', None)
+		if self.description is not None:
+			self.__doc__ = self.description
 		
 		for key in raw:
 			if key not in self.__dict__:
 				prt.info(f'Found optional project info: {key}')
 				setattr(self, key, raw[key])
 		
-		self.init = raw.get('init', None)
-		self.init_name = raw.get('init_name', None)
 		
 		# endregion
 		
@@ -716,18 +737,17 @@ class Project(Workspace):
 		# endregion
 		# region components
 		
-		self.last_update = raw.get('last_update', None)
+		# self.last_update = raw.get('last_update', None)
 		self.conda_env = raw.get('conda', None)  # TODO
 		
+		self.package = raw.get('package', None)
+		
 		self.no_auto_config = raw.get('no_auto_config', None)
-		if self.no_auto_config is None or not self.no_auto_config:
-			contents = set(os.listdir(self.root))
-			auto_names = {'config', 'configs'}
-			for aname in auto_names:
-				if aname in contents:
-					auto_config = os.path.join(self.root, aname)
-					if os.path.isdir(auto_config) and auto_config not in self.config_paths:
-						self.config_paths.append(auto_config)
+		if not self.no_auto_config:
+			for aname in ['config', 'configs']:
+				path = self.root / aname
+				if path.is_dir() and str(path) not in self.config_paths:
+					self.config_paths.append(str(path))
 		
 		# endregion
 		
@@ -737,18 +757,27 @@ class Project(Workspace):
 		root = self.get_path()
 		
 		if self.add_to_path:
-			sys.path.append(root)
+			sys.path.append(str(root))
 		
 		origin = os.getcwd()
-		os.chdir(root)
+		os.chdir(str(root))
 		
 		super().initialize()
 		
-		# if self.init is not None:
-		# 	name = self.get_name() if self.init_name is None else self.init_name
-		# 	include_project(name, self.init)
-		
+		name = self.get_name()
+		if self.package is not None and name not in sys.modules:
+			path = root / self.package / '__init__.py'
+			if path.exists():
+				spec = importlib.util.spec_from_file_location(name, str(path))
+				mod = importlib.util.module_from_spec(spec)
+				sys.modules[mod.__name__] = mod
+				projects_module.__dict__[mod.__name__] = mod
+				spec.loader.exec_module(mod)
+				# mod.MyClass()
+			
 		os.chdir(origin)
+		
+		prt.info(f'Project "{name}" initialized')
 	
 	# region Registration
 	
@@ -795,6 +824,7 @@ class Project(Workspace):
 		try:
 			path = super().find_config(name)
 		except MissingConfigError:
+			raise
 			path = self.get_path() / Path(name)
 			if path.suffix in {'.yaml', '.yml'} and path.is_file():
 				return str(path)
